// src/planetscale.ts
import * as planetScale from "@planetscale/database";
import { Debug, DriverAdapterError } from "@prisma/driver-adapter-utils";
import { Mutex } from "async-mutex";

// package.json
var name = "@prisma/adapter-planetscale";

// src/conversion.ts
import { cast as defaultCast } from "@planetscale/database";
import { ColumnTypeEnum } from "@prisma/driver-adapter-utils";

// src/text.ts
var decoder = new TextDecoder("utf-8");
function decodeUtf8(text) {
  return text ? decoder.decode(uint8Array(text)) : "";
}
function uint8Array(text) {
  return Uint8Array.from(bytes(text));
}
function bytes(text) {
  return text.split("").map((c) => c.charCodeAt(0));
}

// src/conversion.ts
function fieldToColumnType(field) {
  switch (field) {
    case "INT8":
    case "UINT8":
    case "INT16":
    case "UINT16":
    case "INT24":
    case "UINT24":
    case "INT32":
    case "YEAR":
      return ColumnTypeEnum.Int32;
    case "UINT32":
    case "INT64":
    case "UINT64":
      return ColumnTypeEnum.Int64;
    case "FLOAT32":
      return ColumnTypeEnum.Float;
    case "FLOAT64":
      return ColumnTypeEnum.Double;
    case "TIMESTAMP":
    case "DATETIME":
      return ColumnTypeEnum.DateTime;
    case "DATE":
      return ColumnTypeEnum.Date;
    case "TIME":
      return ColumnTypeEnum.Time;
    case "DECIMAL":
      return ColumnTypeEnum.Numeric;
    case "CHAR":
    case "TEXT":
    case "VARCHAR":
      return ColumnTypeEnum.Text;
    case "ENUM":
      return ColumnTypeEnum.Enum;
    case "JSON":
      return ColumnTypeEnum.Json;
    case "BLOB":
    case "BINARY":
    case "VARBINARY":
    case "BIT":
    case "BITNUM":
    case "HEXNUM":
    case "HEXVAL":
    case "GEOMETRY":
      return ColumnTypeEnum.Bytes;
    case "NULL":
      return ColumnTypeEnum.Int32;
    default:
      throw new Error(`Unsupported column type: ${field}`);
  }
}
var cast = (field, value) => {
  if (field.type === "JSON") {
    return typeof value === "string" && value.length ? decodeUtf8(value) : value;
  }
  const defaultValue = defaultCast(field, value);
  if (defaultValue instanceof Uint8Array) {
    return Array.from(defaultValue);
  }
  return defaultValue;
};

// src/deferred.ts
function createDeferred() {
  const deferred = {};
  return [
    deferred,
    new Promise((resolve, reject) => {
      deferred.resolve = resolve;
      deferred.reject = reject;
    })
  ];
}

// src/errors.ts
function convertDriverError(error) {
  switch (error.code) {
    case 1062: {
      const index = error.message.split(" ").pop()?.split("'").at(1)?.split(".").pop();
      return {
        kind: "UniqueConstraintViolation",
        constraint: index !== void 0 ? { index } : void 0
      };
    }
    case 1451:
    case 1452: {
      const field = error.message.split(" ").at(17)?.split("`").at(1);
      return {
        kind: "ForeignKeyConstraintViolation",
        constraint: field !== void 0 ? { fields: [field] } : void 0
      };
    }
    case 1263: {
      const index = error.message.split(" ").pop()?.split("'").at(1);
      return {
        kind: "NullConstraintViolation",
        constraint: index !== void 0 ? { index } : void 0
      };
    }
    case 1264:
      return {
        kind: "ValueOutOfRange",
        cause: error.message
      };
    case 1364:
    case 1048: {
      const field = error.message.split(" ").at(1)?.split("'").at(1);
      return {
        kind: "NullConstraintViolation",
        constraint: field !== void 0 ? { fields: [field] } : void 0
      };
    }
    case 1049: {
      const db = error.message.split(" ").pop()?.split("'").at(1);
      return {
        kind: "DatabaseDoesNotExist",
        db
      };
    }
    case 1007: {
      const db = error.message.split(" ").at(3)?.split("'").at(1);
      return {
        kind: "DatabaseAlreadyExists",
        db
      };
    }
    case 1044: {
      const db = error.message.split(" ").pop()?.split("'").at(1);
      return {
        kind: "DatabaseAccessDenied",
        db
      };
    }
    case 1045: {
      const user = error.message.split(" ").at(4)?.split("@").at(0)?.split("'").at(1);
      return {
        kind: "AuthenticationFailed",
        user
      };
    }
    case 1146: {
      const table = error.message.split(" ").at(1)?.split("'").at(1)?.split(".").pop();
      return {
        kind: "TableDoesNotExist",
        table
      };
    }
    case 1054: {
      const column = error.message.split(" ").at(2)?.split("'").at(1);
      return {
        kind: "ColumnNotFound",
        column
      };
    }
    case 1406: {
      const column = error.message.split(" ").flatMap((part) => part.split("'")).at(6);
      return {
        kind: "LengthMismatch",
        column
      };
    }
    case 1191:
      return {
        kind: "MissingFullTextSearchIndex"
      };
    case 1213:
      return {
        kind: "TransactionWriteConflict"
      };
    case 1040:
    case 1203:
      return {
        kind: "TooManyConnections",
        cause: error.message
      };
    default:
      return {
        kind: "mysql",
        code: error.code,
        message: error.message,
        state: error.state
      };
  }
}

// src/planetscale.ts
var debug = Debug("prisma:driver-adapter:planetscale");
var RollbackError = class _RollbackError extends Error {
  constructor() {
    super("ROLLBACK");
    this.name = "RollbackError";
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, _RollbackError);
    }
  }
};
var PlanetScaleQueryable = class {
  constructor(client) {
    this.client = client;
  }
  provider = "mysql";
  adapterName = name;
  /**
   * Execute a query given as SQL, interpolating the given parameters.
   */
  async queryRaw(query) {
    const tag = "[js::query_raw]";
    debug(`${tag} %O`, query);
    const { fields, insertId: lastInsertId, rows } = await this.performIO(query);
    const columns = fields.map((field) => field.name);
    return {
      columnNames: columns,
      columnTypes: fields.map((field) => fieldToColumnType(field.type)),
      rows,
      lastInsertId
    };
  }
  /**
   * Execute a query given as SQL, interpolating the given parameters and
   * returning the number of affected rows.
   * Note: Queryable expects a u64, but napi.rs only supports u32.
   */
  async executeRaw(query) {
    const tag = "[js::execute_raw]";
    debug(`${tag} %O`, query);
    return (await this.performIO(query)).rowsAffected;
  }
  /**
   * Run a query against the database, returning the result set.
   * Should the query fail due to a connection error, the connection is
   * marked as unhealthy.
   */
  async performIO(query) {
    const { sql, args: values } = query;
    try {
      const result = await this.client.execute(sql, values, {
        as: "array",
        cast
      });
      return result;
    } catch (e) {
      const error = e;
      onError(error);
    }
  }
};
function onError(error) {
  if (error.name === "DatabaseError") {
    const parsed = parseErrorMessage(error.message);
    if (parsed) {
      throw new DriverAdapterError(convertDriverError(parsed));
    }
  }
  debug("Error in performIO: %O", error);
  throw error;
}
function parseErrorMessage(error) {
  const regex = /^(.*) \(errno (\d+)\) \(sqlstate ([A-Z0-9]+)\)/;
  let match = null;
  while (true) {
    const result = error.match(regex);
    if (result === null) {
      break;
    }
    match = result;
    error = match[1];
  }
  if (match !== null) {
    const [, message, codeAsString, sqlstate] = match;
    const code = Number.parseInt(codeAsString, 10);
    return {
      message,
      code,
      state: sqlstate
    };
  } else {
    return void 0;
  }
}
var LOCK_TAG = Symbol();
var PlanetScaleTransaction = class extends PlanetScaleQueryable {
  constructor(tx, options, txDeferred, txResultPromise) {
    super(tx);
    this.options = options;
    this.txDeferred = txDeferred;
    this.txResultPromise = txResultPromise;
  }
  // The PlanetScale connection objects are not meant to be used concurrently,
  // so we override the `performIO` method to synchronize access to it with a mutex.
  // See: https://github.com/mattrobenolt/ps-http-sim/issues/7
  [LOCK_TAG] = new Mutex();
  async performIO(query) {
    const release = await this[LOCK_TAG].acquire();
    try {
      return await super.performIO(query);
    } catch (e) {
      onError(e);
    } finally {
      release();
    }
  }
  async commit() {
    debug(`[js::commit]`);
    this.txDeferred.resolve();
    return await this.txResultPromise;
  }
  async rollback() {
    debug(`[js::rollback]`);
    this.txDeferred.reject(new RollbackError());
    return await this.txResultPromise;
  }
};
var PrismaPlanetScaleAdapter = class extends PlanetScaleQueryable {
  constructor(client) {
    super(client);
  }
  executeScript(_script) {
    throw new Error("Not implemented yet");
  }
  getConnectionInfo() {
    const url = this.client.connection()["url"];
    const dbName = new URL(url).pathname.slice(1);
    return {
      schemaName: dbName || void 0,
      // If `dbName` is an empty string, do not set a schema name
      supportsRelationJoins: true
    };
  }
  async startTransaction(isolationLevel) {
    const options = {
      usePhantomQuery: true
    };
    const tag = "[js::startTransaction]";
    debug("%s options: %O", tag, options);
    const conn = this.client.connection();
    if (isolationLevel) {
      await conn.execute(`SET TRANSACTION ISOLATION LEVEL ${isolationLevel}`).catch((error) => onError(error));
    }
    return this.startTransactionInner(conn, options);
  }
  async startTransactionInner(conn, options) {
    return new Promise((resolve, reject) => {
      const txResultPromise = conn.transaction(async (tx) => {
        const [txDeferred, deferredPromise] = createDeferred();
        const txWrapper = new PlanetScaleTransaction(tx, options, txDeferred, txResultPromise);
        resolve(txWrapper);
        return deferredPromise;
      }).catch((error) => {
        if (!(error instanceof RollbackError)) {
          return reject(error);
        }
        return void 0;
      });
    });
  }
  async dispose() {
  }
};
var PrismaPlanetScaleAdapterFactory = class {
  constructor(config) {
    this.config = config;
  }
  provider = "mysql";
  adapterName = name;
  async connect() {
    return new PrismaPlanetScaleAdapter(new planetScale.Client(this.config));
  }
};
export {
  PrismaPlanetScaleAdapterFactory as PrismaPlanetScale
};
